<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Repositories\IncidenceCategoryRepository;
use App\Repositories\IncidenceStudentRepository;
use Laravel\Lumen\Routing\Controller as BaseController;

/**
 * Controlador para Incidencia Categoria
 * @since   2018-03-13
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2018-03-13<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del controlador<br/>
 *          #edit1
 */
class IncidenceCategoryController extends BaseController {

    protected $incidenceCategory;
    protected $incidenceStudent;

   /**
     * Constructor
     * @since   2018-03-13
     * @param   IncidenceCategoryRepository  $incidenceCategory repositorio
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     *
     */
    public function __construct(IncidenceCategoryRepository $incidenceCategory, IncidenceStudentRepository $incidenceStudent){
        $this->incidenceCategory = $incidenceCategory;
        $this->incidenceStudent = $incidenceStudent;
    }


    /**
     * Método que devuelve el la lista de relaciones de incidencia
     * @since   2018-03-13
     * @author  Erick Vaca <epvaca@rutatec.com><br /><br/>
     *
     * @param   void
     * @return  array types attendance
     * 
     * @edit    2018-03-13<br />
     *          Erick Vaca <epvaca@rutatec.com><br /><br/>
     *          Creación del método<br/>
     *          #edit1
     */
    public function postIncidenceAttendanceRelations() {
        $attendanceRelations = $this->incidenceCategory->incidenceAttendanceRelations();
        return response()->json($attendanceRelations);
    }


  /**
     * Método que guarda categoria
     * @since   2018-03-13
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   array $request
     * @return  
     * 
     */
    public function postSaveIncidenceCategory(Request $request) {
        try {
            //Procesamiento categoria Padre
            $this->validate($request, [
                'serial_scy' => 'required',                
                'attendance_relation_ica' => 'required',               
            ]);
            $result = $this->incidenceCategory->saveIncidenceCategory($request);

            //Procesamiento categorias hijas de existir
            $request['subcategories'] = json_decode($request['subcategories'],true);
            if (is_array($request['subcategories']) && sizeof($request['subcategories']) > 0) {
                foreach ($request['subcategories'] as $sub) {
                    if(isset($sub['serial_ica'])){                        
                        $sub['serial_scy'] = $result->serial_scy;   
                    }else{
                        $sub['serial_ica'] = null;
                        $sub['incidence_serial_ica'] = $result->serial_ica;
                        $sub['serial_scy'] = $result->serial_scy;   
                    }
                                        
                    $resultHijo = $this->incidenceCategory->saveIncidenceCategory($sub);
                }

            }
            
            return response()->json($result);    
        } catch (Exception $ex) {
            return false;
        }
    }


    /**
     * Método para obtener categorias por anio
     * @since   2018-03-13
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_scy serial del anio
     * @return  Categories array
     * 
     */
    public function getIncidenceCategoriesByYear($serial_scy) {
        $categories = $this->incidenceCategory->incidenceCategoriesByYear($serial_scy);
        return response()->json($categories);
    }

    /**
     * Método para obtener subcategorias por anio
     * @since   2018-03-17
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_scy serial del anio
     * @return  Categories array
     * 
     */
    public function getIncidenceSubcategoriesByYear($serial_scy) {
        $categories = $this->incidenceCategory->incidenceSubcategoriesByYear($serial_scy);
        return response()->json($categories);
    }


    /**
     * Método para obtener subcategorias por anio que tengan configuracion
     * @since   2018-03-24
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_scy serial del anio
     * @return  Categories array
     * 
     */
    public function getIncidenceSubcategoriesByYearWithSetup($serial_scy) {
        $categories = $this->incidenceCategory->incidenceSubcategoriesByYearWithSetup($serial_scy);
        return response()->json($categories);
    }


    /**
     * Método para obtener informacion de categoria por serial_ica
     * @since   2018-04-02
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_ica serial de categoria
     * @return  Categories array
     * 
     */
    public function getIncidenceCategoryBySerialIca($serial_ica) {
        $category = $this->incidenceCategory->getIncidenceCategoryBySerialIca($serial_ica);
        return response()->json($category);
    }

    /**
     * Método para obtener total de incidencias/estudiante registradas por serial_ica
     * @since   2018-04-14
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_ica serial de categoria
     * @return  Categories array
     * 
     */
    public function getIncidenceStudentTotalsBySerialIca($serial_ica) {
        $totals = $this->incidenceStudent->getIncidenceStudentTotalsBySerialIca($serial_ica);
        return response()->json($totals);
    }

    /**
     * Método para obtener total de incidencias/estudiante registradas por serial_ica
     * @since   2018-04-14
     * @author  Jairo Serrano <jaserrano@rutatec.com>
     *
     * @param   int $serial_ica serial de categoria
     * @return  Categories array
     *
     */
    public function getIncidenceStudentTotalsBySerialIcaAndStudent($serial_ica,$serial_std,$serial_sbp) {
        $totals = $this->incidenceStudent->getIncidenceStudentTotalsBySerialIcaAndStudent($serial_ica,$serial_std,$serial_sbp);
        return response()->json($totals);
    }

    /**
     * Método que obtiene total de registros de incidencias por serial_ica (padre/categoria).
     * Usado para validar en la configuracion si se puede editar o no categorias
     * @since   2018-04-15
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *     
     * @param   $serial_ica serial_ica     
     * @return  
     * 
     */
    public function getIncidenceStudentTotalsBySerialIcaParent($serial_ica) {
        $totals = $this->incidenceStudent->getIncidenceStudentTotalsBySerialIcaParent($serial_ica);
        return response()->json($totals);
    }
    

}