<?php

namespace App\Repositories;

use App\Models\Event;
use App\Models\EventProfile;
use App\Models\EventSection;
use App\Repositories\BaseRepository;

class EventRepository extends BaseRepository {

    protected $modelClass = Event::class;
    protected $event;

    //Declaración del constructor
    public function __construct(Event $event) {
        $this->event = $event;
    }

    /**
     * Devuelve los eventos pertenecientes a un perfil
     *
     * @param   int $serial_prf serial del perfill
     * 
     * @return  array $events
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    public function getEventProfile($serial_prf) {
        $query = $this->newQuery();
        $query->join('event_profile', 'event.serial_event', '=', 'event_profile.serial_event');
        $query->leftjoin('event_section', function ($join) {
            $join->on('event.serial_event', '=', 'event_section.serial_event');
            $join->where('status_evset', '=', 'ACTIVE');
        });
        $query->leftjoin('section', function ($join) {
            $join->on('section.serial_set', '=', 'event_section.serial_set');
            $join->where('status_set', '=', 'ACTIVE');
        });
        $query->where('status_evpf', '=', 'ACTIVE');
        $query->where('status_event', '=', 'ACTIVE');
        if ($serial_prf != 'all') {
            $query->where('event_profile.serial_prf', '=', $serial_prf);
        }
        $query->selectRaw('event.serial_event as id,name_event as title,start_date_event as start,end_date_event as end,color_set as color,IF(count(section.serial_set)>1,"#3a87ad",color_set) as color');
        $query->groupBy('id', 'serial_prf');
        $query->distinct();
        return $this->doQueryGet($query)->toArray();
    }

    /**
     * Guarda un evento creado
     *
     * @param   array $data datos del nuevo evento a crear
     * 
     * @return  string success
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    public function saveEvent($data) {
        try {
            $event = new Event;
            $event->name_event = $data['name_event'];
            $event->description_event = $data['description_event'];
            $event->start_date_event = $data['start_date_event'];
            $event->end_date_event = $data['end_date_event'];
            $event->save();
            $data['profiles'] = json_decode($data['profiles']);
            foreach ($data['profiles'] as $profile) {
                $event_profile = new EventProfile;
                $event_profile->serial_prf = $profile;
                $event_profile->serial_event = $event->serial_event;
                $event_profile->save();
            }
            if ($data['sections'] != false) {
                $data['sections'] = json_decode($data['sections']);
                foreach ($data['sections'] as $section) {
                    $event_section = new EventSection;
                    $event_section->serial_event = $event->serial_event;
                    $event_section->serial_set = $section;
                    $event_section->save();
                }
            }

            return 'success';
        } catch (Exception $ex) {
            return 'error';
        }
    }

    /**
     * Devuelve los estados del evento
     *
     * @param   void
     * 
     * @return  array $status_event
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    public function getStatusEvent() {
        return $status_event = Event::$status_event;
    }

    /**
     * Devuelve los perfiles asignados a un determinado evento
     *
     * @param   int $serial_event serial del evento
     * 
     * @return  array
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    public function getEventProfiles($serial_event) {
        $event = Event::find($serial_event);
        return $event->profiles;
    }

    /**
     * actualización de datos del evento
     *
     * @param   array $data datos del evento a actualizar
     * 
     * @return  string success
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    public function editEvent($data)
    {
        try {
            $event = Event::find($data['serial_event']);
            $event->name_event = $data['name_event'];
            $event->description_event = $data['description_event'];
            $event->start_date_event = $data['start_date_event'];
            $event->end_date_event = $data['end_date_event'];
            $event->status_event = $data['status_event'];
            $event->save();
            if (isset($data['sections'])) {
                $data['sections'] = json_decode($data['sections']);
                $this->updateEventSections($data['serial_event'], $data['sections'], $data['status_event']);
            }
            $data['profiles'] = json_decode($data['profiles']);
            $this->updateEventProfiles($data['serial_event'], $data['profiles'], $data['status_event']);
            return 'success';
        } catch (Exception $ex) {
            return 'error';
        }
    }

    /**
     * actualización de perfiles de un determinado evento
     *
     * @param   int $serial_event serial del evento
     * @param    int $profiles perfiles que van a ser asignados al evento
     * @param   string $status_event estado del evento
     * @return  void
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    private function updateEventProfiles($serial_event, $profiles, $status_event) {
        if ($status_event == 'ACTIVE') {
            foreach ($profiles as $profile) {
                $eventProfile = $this->newQuery();
                $eventProfile->join('event_profile', 'event_profile.serial_event', '=', 'event.serial_event');
                $eventProfile->where('status_event', '=', 'ACTIVE');
                $eventProfile->where('event_profile.serial_event', '=', $serial_event);
                $eventProfile->where('serial_prf', '=', $profile);
                $result = $this->doQueryFirst($eventProfile);
                if ($result == null) {
                    $eventPrf = new EventProfile;
                    $eventPrf->serial_event = $serial_event;
                    $eventPrf->serial_prf = $profile;
                    $eventPrf->save();
                } elseif ($result->status_evpf == 'INACTIVE') {
                    $eventProf = EventProfile::where('serial_event', '=', $serial_event)
                                    ->where('serial_prf', '=', $result->serial_prf)->first();
                    $eventProf->status_evpf = 'ACTIVE';
                    $eventProf->save();
                }
            }
            $profilesToInactive = $this->newQuery();
            $profilesToInactive->join('event_profile', 'event_profile.serial_event', '=', 'event.serial_event');
            $profilesToInactive->where('status_event', '=', 'ACTIVE');
            $profilesToInactive->where('event_profile.serial_event', '=', $serial_event);
            $profilesToInactive->whereNotIn('serial_prf', $profiles);
            $toInactive = $this->doQueryGet($profilesToInactive);
            foreach ($toInactive as $inactive) {
                $eventProfile = EventProfile::where('serial_event', '=', $serial_event)
                                ->where('serial_prf', '=', $inactive->serial_prf)->first();
                $eventProfile->status_evpf = 'INACTIVE';
                $eventProfile->save();
            }
        } else {
            $event = Event::find($serial_event);
            $event->status_event = 'INACTIVE';
            $event->save();
            $event_profiles = EventProfile::where('serial_event', '=', $serial_event)->get();
            foreach ($event_profiles as $eventPrf) {
                $eventPrf->status_evpf = 'INACTIVE';
                $eventPrf->save();
            }
        }
    }

    /**
     * obtener todas las secciones que pertenecen a un evento
     *
     * @param   int $serial_event serial del evento
     * @return  array $sections
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    public function getEventSections($serial_event) {
        $event = Event::find($serial_event);
        return $event->sections;
    }

    /**
     * actualización de secciones de un determinado evento
     *
     * @param   int $serial_event serial del evento
     * @param   array $sections seciones que van a ser asignados al evento
     * @param   string $status_event estado del evento
     * @return  void
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    private function updateEventSections($serial_event, $sections, $status_event) {
        if ($status_event == 'ACTIVE') {
            foreach ($sections as $section) {
                $eventSection = $this->newQuery();
                $eventSection->join('event_section', 'event_section.serial_event', '=', 'event.serial_event');
                $eventSection->where('status_event', '=', 'ACTIVE');
                $eventSection->where('event_section.serial_event', '=', $serial_event);
                $eventSection->where('serial_set', '=', $section);
                $result = $this->doQueryFirst($eventSection);
                if ($result == null) {
                    $eventSet = new EventSection;
                    $eventSet->serial_event = $serial_event;
                    $eventSet->serial_set = $section;
                    $eventSet->save();
                } elseif ($result->status_evset == 'INACTIVE') {
                    $eventSect = EventSection::where('serial_event', '=', $serial_event)
                                    ->where('serial_set', '=', $result->serial_set)->first();
                    $eventSect->status_evset = 'ACTIVE';
                    $eventSect->save();
                }
            }
            $sectionsToInactive = $this->newQuery();
            $sectionsToInactive->join('event_section', 'event_section.serial_event', '=', 'event.serial_event');
            $sectionsToInactive->where('status_event', '=', 'ACTIVE');
            $sectionsToInactive->where('event_section.serial_event', '=', $serial_event);
            $sectionsToInactive->whereNotIn('serial_set', $sections);
            $toInactive = $this->doQueryGet($sectionsToInactive);
            foreach ($toInactive as $inactive) {
                $eventSection = EventSection::where('serial_event', '=', $serial_event)
                                ->where('serial_set', '=', $inactive->serial_set)->first();
                $eventSection->status_evset = 'INACTIVE';
                $eventSection->save();
            }
        } else {
            $event = Event::find($serial_event);
            $event->status_event = 'INACTIVE';
            $event->save();
            $event_sections = EventSection::where('serial_event', '=', $serial_event)->get();
            foreach ($event_sections as $eventSet) {
                $eventSet->status_evset = 'INACTIVE';
                $eventSet->save();
            }
        }
    }

}
