<?php

namespace App\Repositories;

use App\Models\OptionalSubject;
use App\Repositories\BaseRepository;

/**
 * Repostiorio para materias optativas
 * @since   2017-07-09
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2017-07-09<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del archivo<br/>
 *          #edit1
 */
class OptionalSubjectRepository extends BaseRepository {

    protected $modelClass = OptionalSubject::class;

//Declaración del constructor
    public function __construct(OptionalSubject $optionalSubject) {
        $this->optionalSubject = $optionalSubject;
    }

    /**
     * Método que retorna materias optativas por seccion, nivel , especialidad, anio lectivo
     * @since   2017-07-16
     * @author  Erick Vaca <epvaca@rutatec.com> 
     * @param   integer serial_scy año lectivo
     * @param   integer $serial_sec seccion
     * @param   integer $serial_spt especialidad
     * @param   integer $serial_lvl nivel
     * @return  ListObject OptionalSubject
     * 
     */
    public function getSubjectsBySchoolYearSectionLevel($serial_scy, $serial_set, $serial_lvl, $serial_spt) {
        $query = $this->newQuery();
        $query->join('period_group_year AS pgy', 'pgy.serial_pgy', '=', 'optional_subject.serial_pgy');
        $query->join('optional_subject_level AS osl', 'osl.serial_osb', '=', 'optional_subject.serial_osb');
        $query->join('cat_subject AS csb', 'csb.serial_csb', '=', 'optional_subject.serial_csb');
        $query->leftJoin('cat_subject as csbj', function($join) {
            $join->on('csbj.serial_csb', '=', 'csb.cat_serial_csb');
        });

        $query->where('optional_subject.status_osb', '=', 'ACTIVE');
        $query->where('pgy.status_pgy', '=', 'ACTIVE');
        $query->where('osl.status_osl', '=', 'ACTIVE');
        $query->where('csb.status_csb', '=', 'ACTIVE');
        $query->where('osl.serial_scy', '=', $serial_scy);
        $query->where('osl.serial_set', '=', $serial_set);
        $query->where('osl.serial_lvl', '=', $serial_lvl);
        $query->where('osl.serial_spt', '=', $serial_spt);

        $query->selectRaw("DISTINCT (optional_subject.serial_osb) as serial, IFNULL(CONCAT(csbj.name_csb, ' - ', csb.name_csb), csb.name_csb) AS name_csb, csb.code_csb, csb.serial_csb");


        return $this->doQueryGet($query);
    }

    /**
     * Método que devuelve los estuduantes de la materia seleccionada
     * @since   2017-08-03
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     * @param   array $serial_osb serial de la materia
     * @param date $date fecha seleccionada
     * @return  ListObject $students
     * 
     * @edit    2017-08-03<br />
     *          Fernando Salas <rfsalas@rutatec.com><br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function getStudents($serial_osb, $date, $serial_cdt = false, $day_ddc = false) {
        $query = $this->newQuery();
        $query->join('student_optional_subject AS sos', 'sos.serial_osb', '=', 'optional_subject.serial_osb');
        $query->join('level_optional_group AS lop', 'sos.serial_lop', '=', 'lop.serial_lop');
        $query->join('student AS std', 'sos.serial_std', '=', 'std.serial_std');
        $query->join('student_by_course AS sbc', 'sbc.serial_std', '=', 'std.serial_std');
        $query->join('course AS crs', 'crs.serial_crs', '=', 'sbc.serial_crs');
        $query->join('period', function($join)use($date) {
            $join->on('period.serial_prd', '=', 'sos.serial_prd');
            $join->where('start_date_prd', '<=', $date);
            $join->where('end_date_prd', '>=', $date);
        });
        if ($serial_cdt) {
            $query->join('schedule AS scd', function($join) {
                $join->on('scd.serial_crs', '=', 'sbc.serial_crs');
                $join->on('scd.serial_lop', '=', 'lop.serial_lop');
            });
            $query->where('serial_cdt', $serial_cdt);
            $query->where('day_scd', $day_ddc);
            $query->where('status_scd', 'ACTIVE');
        }
        $query->where('sos.serial_osb', $serial_osb);
        $query->where('optional_subject.status_osb', 'ACTIVE');
        $query->where('sos.status_sos', 'ACTIVE');
        $query->where('lop.status_lop', 'ACTIVE');
        $query->where('std.status_std', 'ACTIVE');
        $query->where('sbc.status_stc', 'ACTIVE');
        $query->where('crs.status_crs', 'ACTIVE');
        $query->whereRaw('crs.serial_scy=lop.serial_scy');
        $query->orderBy('std.last_name_std', 'ASC');
        $query->orderBy('std.first_name_std', 'ASC');
        return $this->doQueryGet($query);
    }

    /**
     * Método que retorna informacion de la materia optativa
     * @since   2018-03-19
     * @author  Erick Vaca <epvaca@rutatec.com> 
     * @param   integer serial_osb serial materia
     * @return  ListObject OptionalSubject
     * 
     */
    public function getOptionalSubjectInformationBySerialOsb($serial_osb) {
        $query = $this->newQuery();
        $query->join('period_group_year AS pgy', 'pgy.serial_pgy', '=', 'optional_subject.serial_pgy');
        $query->join('optional_subject_level AS osl', 'osl.serial_osb', '=', 'optional_subject.serial_osb');

        $query->where('optional_subject.status_osb', '=', 'ACTIVE');
        $query->where('optional_subject.serial_osb', '=', $serial_osb);
        $query->where('pgy.status_pgy', '=', 'ACTIVE');
        $query->where('osl.status_osl', '=', 'ACTIVE');

        return $this->doQueryGet($query)->toArray();
    }

    /**
     * Método que devuelve la materia, su sección,nivel y paralelo
     * @since   2018-03-22
     * @author  Erick Vaca <epvaca@rutatec.com> 
     * @param   integer  $serial_osb serial de la materia
     * @param   integer  $serial_scy serial del año lectivo
     * @return  object materia     
     */
    public function getOptionalSubjectBySerialOsbSchoolYear($serial_osb, $serial_scy) {

        $query = $this->newQuery();

        $query->join('optional_subject_level as osl', 'optional_subject.serial_osb', '=', 'osl.serial_osb');
        $query->join('level as lvl', 'lvl.serial_lvl', '=', 'osl.serial_lvl');
        $query->join('section as set', 'set.serial_set', '=', 'osl.serial_set');
        $query->join('period_group_year as pgy', 'pgy.serial_pgy', '=', 'optional_subject.serial_pgy');
        $query->join('school_year as scy', 'scy.serial_scy', '=', 'pgy.serial_scy');
        $query->join('cat_subject as csb', 'csb.serial_csb', '=', 'optional_subject.serial_csb');
        $query->LeftJoin('cat_subject as csbj', 'csbj.serial_csb', '=', 'csb.cat_serial_csb');
        $query->where('optional_subject.status_osb', 'ACTIVE');
        $query->where('osl.status_osl', 'ACTIVE');
        $query->where('pgy.status_pgy', 'ACTIVE');
        $query->where('csb.status_csb', 'ACTIVE');
        $query->where('scy.serial_scy', $serial_scy);
        $query->where('optional_subject.serial_osb', $serial_osb);
        $query->selectRaw("scy.name_scy,set.name_set,' ' as course,optional_subject.serial_osb as serial_sbc,pgy.serial_pgy,IFNULL(CONCAT(csbj.name_csb, ' - ', csb.name_csb),
            CONCAT(csb.name_csb,' (',lvl.description_lvl,')')) AS name_csb,scy.setup_skills");
        return $this->doQueryFirst($query);
    }

    /**
     * Método que devuelve el detalle de la materia optativa
     * @since   2018-06-08
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     * @param   integer $serial_osb serial de la materia
     * @return  object $optionalSubject
     *
     * @edit    2017-08-03<br />
     *          Fernando Salas <rfsalas@rutatec.com><br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function getOptionalSubject($serial_osb) {
        $optionalSubject = $this->newQuery();
        $optionalSubject->join('optional_subject_level as osl', function ($join) {
            $join->on('osl.serial_osb', '=', 'optional_subject.serial_osb');
            $join->where('status_osl', '=', 'ACTIVE');
        });
        $optionalSubject->join('group_optional_subject as gos', function ($join) {
            $join->on('optional_subject.serial_osb', '=', 'gos.serial_osb')
                    ->where('status_gos', '=', 'ACTIVE');
        });
        $optionalSubject->join('cat_subject as csb', function ($join) {
            $join->on('optional_subject.serial_csb', '=', 'csb.serial_csb')
                    ->where('status_csb', '=', 'ACTIVE');
        });
        $optionalSubject->leftjoin('subject_category as scat', function ($join) {
            $join->on('osl.serial_scat', '=', 'scat.serial_scat')
                    ->where('status_scat', '=', 'ACTIVE');
        });
        $optionalSubject->where('optional_subject.serial_osb', '=', $serial_osb);
        $optionalSubject->selectRaw('optional_subject.serial_emp,optional_subject.serial_osb as serial,IF(csb.short_name_csb is null,IF(scat.name_scat is null,csb.name_csb,scat.name_scat),csb.short_name_csb) as "name_subject"');
        return $this->doQueryFirst($optionalSubject);
    }

    /**
     * Método que retorna la materia optativa por serial_lop, estudiante, periodo
     * @since   2018-07-23
     * @author  Erick Vaca <epvaca@rutatec.com> 
     * @param   integer serial_lop serial lop
     * @param   integer serial_std serial estudiante
     * @param   integer serial_prd serial periodo
     * @return  ListObject OptionalSubject
     * 
     */
    public function getOptionalSubjectByStudentSerialLopPeriod($serial_lop, $serial_std, $serial_prd) {
        $query = $this->newQuery();
        $query->join('student_optional_subject AS sos', 'sos.serial_osb', '=', 'optional_subject.serial_osb');

        $query->where('optional_subject.status_osb', '=', 'ACTIVE');
        $query->where('sos.serial_lop', '=', $serial_lop);
        $query->where('sos.serial_std', '=', $serial_std);
        $query->where('sos.serial_prd', '=', $serial_prd);

        $query->where('sos.status_sos', '=', 'ACTIVE');

        return $this->doQueryFirst($query);
    }

    /**
     * Método que retorna la materia optativa por serial_lop, estudiante, periodo
     * @since   2018-07-23
     * @author  Erick Vaca <epvaca@rutatec.com> 
     * @param   integer serial_lop serial lop
     * @param   integer serial_std serial estudiante
     * @param   integer serial_prd serial periodo
     * @return  ListObject OptionalSubject
     * 
     */
    public function getOptionalSubjectByStudentSerialLopPeriodDate($serial_lop, $serial_std, $date) {
        $query = $this->newQuery();
        $query->join('student_optional_subject AS sos', 'sos.serial_osb', '=', 'optional_subject.serial_osb');
        $query->join('period AS prd', function($join) use($date) {
            $join->on('sos.serial_prd', '=', 'prd.serial_prd');
            $join->where('prd.start_date_prd', '<=', $date);
            $join->where('prd.end_date_prd', '>=', $date);
        });
        $query->where('optional_subject.status_osb', '=', 'ACTIVE');
        $query->where('sos.serial_lop', '=', $serial_lop);
        $query->where('sos.serial_std', '=', $serial_std);
        $query->where('sos.status_sos', '=', 'ACTIVE');
        return $this->doQueryFirst($query);
    }

    public function getAcademicArea($serial_scy, $serial_set, $serial_lvl) {
        $optional = $this->newQuery();
        $optional->join('optional_subject_level', function ($join) {
            $join->on('optional_subject.serial_osb', '=', 'optional_subject_level.serial_osb');
        });
        $optional->join('cat_subject', function ($join) {
            $join->on('optional_subject.serial_csb', '=', 'cat_subject.serial_csb');
        });
        $optional->join('academic_area', function ($join) {
            $join->on('cat_subject.serial_aca', '=', 'academic_area.serial_aca');
        });
        $optional->where('optional_subject_level.serial_scy', '=', $serial_scy);
        $optional->where('optional_subject_level.serial_set', '=', $serial_set);
        $optional->where('optional_subject_level.serial_lvl', '=', $serial_lvl);
        $optional->where('optional_subject_level.status_osl', '=', 'ACTIVE');
        $optional->selectRaw('distinct(academic_area.serial_aca),academic_area.name_aca');
        return $this->doQueryGet($optional);
    }

}
